package dev.ninjdai.werewolf;

import dev.ninjdai.werewolf.commands.GamemodeCommand;
import dev.ninjdai.werewolf.commands.lg.LGCommand;
import dev.ninjdai.werewolf.commands.PlayerKillCommand;
import dev.ninjdai.werewolf.gui.Gui;
import dev.ninjdai.werewolf.uhc.UHCPlayer;
import dev.ninjdai.werewolf.worldgen.LGGenerator;
import dev.ninjdai.werewolf.worldgen.LGTerrainBuilder;
import dev.ninjdai.werewolf.worldgen.TerrainBuilder;
import io.github.togar2.pvp.MinestomPvP;
import io.github.togar2.pvp.feature.CombatFeatureSet;
import io.github.togar2.pvp.feature.CombatFeatures;
import net.minestom.server.MinecraftServer;
import net.minestom.server.event.GlobalEventHandler;
import net.minestom.server.extras.lan.OpenToLAN;
import net.minestom.server.extras.velocity.VelocityProxy;
import net.minestom.server.instance.InstanceContainer;
import net.minestom.server.instance.InstanceManager;
import net.minestom.server.instance.LightingChunk;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Main {
    public static final Logger LOGGER = LoggerFactory.getLogger("Playground");

    public static InstanceManager instanceManager;
    public static InstanceContainer overworld;

    public static void main(String[] args) {
        MinecraftServer minecraftServer = MinecraftServer.init();

        instanceManager = MinecraftServer.getInstanceManager();
        overworld = instanceManager.createInstanceContainer();

        //VanillaReimplementation vri = VanillaReimplementation.hook(MinecraftServer.process());

        overworld.setChunkSupplier(LightingChunk::new);
        overworld.setGenerator(new LGGenerator(new LGTerrainBuilder(3301)));

        GlobalEventHandler globalEventHandler = MinecraftServer.getGlobalEventHandler();
        EventHandler.register(globalEventHandler);
        globalEventHandler.addChild(Gui.node);

        MinecraftServer.getCommandManager().register(new LGCommand());
        MinecraftServer.getCommandManager().register(new PlayerKillCommand());
        MinecraftServer.getCommandManager().register(new GamemodeCommand());

        MinestomPvP.init();
        CombatFeatureSet legacyVanilla = CombatFeatures.legacyVanilla();
        MinecraftServer.getGlobalEventHandler().addChild(legacyVanilla.createNode());

        MinecraftServer.getConnectionManager().setPlayerProvider(UHCPlayer::new);

        overworld.loadChunk(0, 0);

        OpenToLAN.open();
        MinecraftServer.setBrandName("Playground");
        //VelocityProxy.enable("cMkrrJ8tdUS6");

        minecraftServer.start("0.0.0.0", 30065);
    }
}
